function [Tsol,Psol,Psisol, X,Z] = FTsol(Tin,parameters,nxpad,nypad)
%[T,P,Psi,X,Z] = FTsol(Tin,parameters,nxpad,nypad)
%Outputs matrix-valued solutions for temperature T, pressure P and stream
%function Psi at a grid (X,Z) given a solution vector T indexed as
%described in convection_timestep_v2. parameters is the parameter structure
%used to compute the solution vector T. nxpad and nypad are the number of
%grid points in the x- and z- direction on which the solution is computed;
%these may differ from the number nx and ny of Fourier components used when
%computing the solution. If nxpad < nx or nypad < ny, the solution is
%computed at nx grid points in the horizontal, or ny grid points in the
%vertical, respectively.

%grid information
nx = parameters.nx;
ny = parameters.ny;

if nargin <  3 || isempty(nxpad)
    nxpad = nx;
end
if nargin < 4 || isempty(nypad)
    nypad = ny;
end

%vertical sine basis functions
k_z = (1:ny).'*pi;
z = (0:nypad)/nypad;
phi_z = sin(k_z*z);

%set up T in matrix form
T = reshape(Tin,nx,ny);
%Fourier series
if nxpad > nx
    pad = zeros(floor((nxpad-nx)/2),ny);
    T = [pad; T; pad;];
end
T = ifftshift(T,1);
T = nxpad*ifft(T);
%Fourier series in vertical
Tsol = T*phi_z;
%flip for plotting
Tsol = Tsol.';

%optional pressure and stream function output
if nargout  > 1
    %pressure
    phi_c_z = cos(k_z*z);
    G = parameters.G;
    K_z = parameters.K_z;
    K_x = parameters.K_x;
    P = reshape(-G*K_z*Tin,nx,ny);
    %Fourier series
    if nxpad > nx
        P = [pad; P; pad;];
    end
    P = ifftshift(P,1);
    P = nxpad*ifft(P);
    %Fourier series in vertical
    Psol = P*phi_c_z;
    %flip for plotting
    Psol = Psol.';
    %stream function
    Psisol = reshape(-1i*G*K_x*Tin,nx,ny);   %Fourier series
    if nxpad > nx
        Psisol = [pad; Psisol; pad;];
    end
    Psisol = ifftshift(Psisol,1);
    Psisol = nxpad*ifft(Psisol);
    %Fourier series in vertical
    Psisol = Psisol*phi_z;
    %flip for plotting
    Psisol = Psisol.';
    
end


%create grid if needed
if nargout > 2
    a = parameters.a;
    x = (0:nxpad-1).'*a/nxpad;
    [X,Z] = meshgrid(x,z);
end

end