function fsolution = convection_evolve(T,parameters,dt,nsteps,nskip,symmetrize,test)
%fsolution = convection_evolve(T,parameters,dt,nsteps,nskip,symmetrize)
%Computes nsteps*nskip backward Euler steps with step size dt of the porous
%media convection problem as defined by the zeros of the function
%convection_timestep_v2, with parameter structure parameters and initial
%condition T. Optionally ensures that Fourier components T_mn satisfy
%conjugation requirement T_mn = conj(T_(m0-m)n}, where m0 is the
%zero-wavenumber Fourier component in x.
%
%Input arguments (all optional)
%   T:  initial condition T as a column vector, indexed as specified in
%       convection_timestep_v2. If T is supplied as a scalar, a
%       perturbation of magnitude T is created randomly. If T is not
%       specified (is an empty array) an random pertubation of magnitude
%       1e-2 is created randomly
%   parameters: the parameter structure required by convection_timestep_v2,
%       If parameters is not supplied (is an empty array), the parameter
%       structure is set to the default output of parameter_generate_v2
%   dt: size of time step, defaults to 1e-2
%   nsteps: nsteps*nskip time steps are computed, with nsteps steps being
%       save as output. Defaults to 100
%   nskip:  the code saves every (nskip)th time step as output, defaults to
%       5
%   symmetrize: to prevent the growth of a spurious imaginary part, T can
%       be forced to satisfy the symmetry relation T_mn = conj(T_{-m,n}) at
%       every time step if symmetrize is set to true. Defaults to false
%
%Output argument
%   fsolution:  Output structure with fields
%       parameters: the parameter structure used
%       T:  nx*ny-by nsteps array of column vectors representating
%           solutions of the convection problems at times fsolution.t
%       t:  time stamps for the solution vectors in fsolution.T


if nargin < 3 || isempty(dt)
    dt = 1e-2;
else
end
if nargin < 4 || isempty(nsteps)
    nsteps = 100;
end
if nargin < 5 || isempty(nskip)
    nskip = 5;
end
if nargin < 6 || isempty(symmetrize)
    symmetrize = false;
end
if nargin < 7 || isempty(test)
    test = false;
end
if nargin < 2 || isempty(parameters)
    if test
        parameters = parameter_generate_v2(5,4);
    else
        parameters = parameter_generate_v2;
    end
end

%write time step size into parameter structure
parameters.dt = dt;

nx = parameters.nx;
ny = parameters.ny;
ntot = nx*ny;

%symmetry index
if symmetrize
    symmindex = parameters.symmindex;
end

%initial condition
if nargin == 0 || isempty(T) || length(T) == 1
    if size(T) == [1 1]
        epsilon = T;
    else
        epsilon = 1e-2;
    end
    T0 = epsilon*(rand(nx,ny)-.5);
    T = 1/nx*fft(T0,[],1);
    T = fftshift(T,1);
    if round(nx/2) == nx/2
        T(1,:) = 0; %get rid of highest, asymmetrical mode
    end
    %make column vector
    T = reshape(T,ntot,1);
end
    
%time set
t = 0;

%talkative solver
srchparams.verbose = 0;

if test
    parameters.Tprev = T;
    fsolution = Jacobian_test_single(@convection_steady_Ra,rand(size([T;1])),parameters);
    return
end
    

%initialize output structure
fsolution.parameters = parameters;
fsolution.T = zeros(ntot,nsteps);
fsolution.t = zeros(nsteps,1);

%set counter
ii=1;
jj=0;

% run through time stepping
while ii < nsteps
    if ii==1 && jj == 0
        fsolution.T(:,ii) = T;  %save initial condition
    end
    t = t+dt;               %increment time stamp
    parameters.Tprev = T;  %previous time step solution for backward Euler step
    T = Newton_single(@convection_timestep_v2,T,parameters,srchparams);
    if symmetrize
        Tconj= conj(T);
        T = 0.5*(T+Tconj(symmindex));
    end 
    if jj < nskip
        jj = jj+1;
    else
        jj = 0;
        ii = ii+1;
        fsolution.T(:,ii) = T;
        fsolution.t(ii) = t;
        disp('step'), ii
    end
end

end