function [fout, Dfout] = convection_steady_Ra_real(Cin,parameters)
%[fout, Dfout] = convection_steady_Ra_real(Cin,parameters)
%   Evaluates residual fout of steady state convection equations defined in
%   convection_steady_Ra and its derivative Dfout. converts from real and
%   imaginary parts of the Fourier coefficients encoded in Cin as the
%   independent variables to the complex Fourier coefficients used by
%   convection_steady_Ra.m using the realindex.realflag,
%   realindex.realtcomplex and symmindex arrays defined by convind_v2.m,
%   see the documentation to convind_v2. Cin concatenates the real and
%   imaginary parts of the Fourier coefficients with the parameter Ra, so
%   that steady state solutions can be computed by arc length continuation.
%   convection_steady_Ra_real also resolves the phase ambiguity of steady
%   state solutions by forcing the Fourier components T_{m0+1,1} and
%   T_{m0-1,1} = conj(T_{m0+1,1}) to have vanishing real parts, see
%   convind_v2 and convection_timestep_v2 for details of the indexing
%   convention used here.
%
%   Input arguments
%       Cin:    concatenated vector vector of real and imaginary parts of
%               Fourier coefficients with Ra. Cin(end) is Ra, while
%               Cin(1:end-1) is the vector of real and imaginary parts of
%               Fourier coefficients. These are related to the  complex
%               Fourier coefficients (as used by convection_evolve.m,
%               convection_timestep_v2.m and convection_steady_Ra.m) by
%               complextorealfun.m using the procedure for mapping from
%               Avec to B defined in the file header of convind_v2.m. If
%               T_hat is the vector of complex Fourier coefficients as used
%               for instance by convection_evolve.m, then the corresponding
%               Cin is computed by 
%                   Cin(1:end-1) = complextorealfun(T_hat,parameters),
%                   Cin(end) = Ra;
%       parameters: parameter structure with the fields output by
%               parameter_generate_v2.m
%   Output arguments
%       fout:   Residual of steady state convection problem. Because the
%               steady state problem has a non-unique answer due to
%               invariance under phase shifts, fout(2) is set equal to 
%       Dfout:  Jacobian of fout with respect to Cin

nx = parameters.nx;
ny = parameters.ny;
realtocomplex = parameters.realindex.realtocomplex;
realflag = parameters.realindex.realflag;
symmindex = parameters.symmindex;

c0 = Cin(1:end-1);
Ra = Cin(end);

nc = length(c0);

Tvecaux = accumarray(realtocomplex,realflag.*c0+1i*(1-realflag).*c0,[nx*ny,1],[],0);
Tvec = 0.5*(Tvecaux + conj(Tvecaux(symmindex)));

if nargout == 1
    faux = convection_steady_Ra([Tvec;Ra],parameters);
else
    [faux, Dfaux] = convection_steady_Ra([Tvec;Ra],parameters);
    DfauxDRa = Dfaux(:,end);
    DfauxDTvec = Dfaux(:,1:end-1);
end
fout = realflag.*real(faux(realtocomplex)) + (1-realflag).*imag(faux(realtocomplex));
fout(2) = c0(2);

if nargout > 1
    symmrealtocomplex = symmindex(realtocomplex);
    DTvecDc = 0.5 * ( sparse(realtocomplex,1:nc,realflag+1i*(1-realflag),nx*ny,nc) + sparse(symmrealtocomplex,1:nc,realflag-1i*(1-realflag),nx*ny,nc) );
    DfauxDc = DfauxDTvec*DTvecDc;
    realdiag = spdiags(realflag,0,length(realflag),length(realflag));
    imagdiag = spdiags(1-realflag,0,length(realflag),length(realflag));
    DfoutDc = realdiag*real(DfauxDc(realtocomplex,:)) + imagdiag*imag(DfauxDc(realtocomplex,:));
    DfoutDRa = realdiag*real(DfauxDRa(realtocomplex)) + imagdiag*imag(DfauxDRa(realtocomplex));
    Dfout = [DfoutDc, DfoutDRa];
    Dfout(2,:) = sparse(1,2,1,1,length(fout)+1);
end

end

