function [Fout,DFout] = convection_timestep_v2(T,parameters)
%[Fout,DFout] = convection_timestep(T,parameters)
%Evaluates residual of backward Euler step function for porous convection
%   (T_mn-T_mn^previous)/dt - T_mn - k_zn * p_mn + conv(k_{zr}T_qr,T_{qr)_mn
%       + conv(k_{xq}p_qr,k_{xq}_T{qr)_mn
%       + conv(k_{zq}T_qr,k_{zq}_p{qr)_mn
%       + 1/Ra*(k_xm^2 + k_zn^2)*ZT_{mn}
%where
%   p_mn = -G_mn*k_zn*T_mn
%and
%   conv(A_{qr}B_{qr})_mn = sum_{q=1}^{nx} sum_{r=1}^{n-1} A_{qr} B_{m-q,n-r}
%       - sum_{q=1}^{nx} sum_{r=n+1}^{ny} A_{qr} B_{m-q,r-n}
%       + sum_{q=1}^{nx} sum_{r=1}^{ny-n} A_{qr} B_{m-q,n+r}
%where 1<m<nx, 1<n<ny. The Green's function transform is
%   G_mn = 1/(k_xm^2+k_yn^2)
%if k_xm^2+k_yn^2 ~= 0, G_mn = 0 otherwise.
%
%The equations above are the Galerkin discretization of the porous
%convection problem
% dT/dt -T + dp/dz + TdT/dz - grad p. grad T - Ra^{-1} div grad T = 0
% dT/dz - div grad p = 0
%Subject to periodic boundary conditions at x = 0,a and
% T = 0
%at z = 0,1
% dp/dz = 0
%at z = 0,1
%
%The Fourier transform indexing is assumed to work as follows: Let
%   m0 = ceil((nx+1)/2);
%then k_xm = 2*(m-m0)*pi/a, where a is lateral period, and the Fourier
%series composition of the solution T(x,z,t) takes the form
% T(x,z,t) = sum_{q=1}^{nx} sum_{r=n+1}^{ny}
% T_{mn}exp(ik_{xn}x)sin(k_{zn}z)
%A practical recipe for computing T(x,z,t) on a grid is given below.
%
%The multi-index mn is converted to a linaer index in the usual way that
%matlab converts matrix subscripts to indices through the function
%ind2sub.m:
%   linear index = n*nx + mx,
%and this is how the input nx*ny-by-1 vector T is constructed
%
%Actual Fourier coefficients can be calculated by
%   T_mat = reshape(T,nx,ny)
%   Tmat = ifftshift(T_mat,1);
%and a p_mat can be constituted similarly. The actual solution can be
%computed on a grid through
%   T_phys = nx*ifft(T_mat);
%   T_phys = T_phys*sin(k_zn*z_vec)
%where $k_zn is a column vector of k_zn values as indicated above
%   k_zn = (1:ny).'*pi/a
%and z_vec a row vector given the the set of values of z at which T is to
%be ebalueated. T_phys_{ij} is then a matrix of values
%   T_phys_ij = T_solution(x_i,z_j)
%where x_i = i*a/nx and y_j = z_vec(j)
%
%convection_timestep_v2 also outputs the Jaocbian of the residual. Note
%that the convolution products conv(A_{qr},B_{qr}) have the following
%symmetry property:
%   conv(B_{qr},A_{qr})_nm = convT(A_{qr}B_{qr})_mn 
%       = sum_{q=1}^{nx} sum_{r=1}^{n-1} A_{qr} B_{m-q,n-r}
%       + sum_{q=1}^{nx} sum_{r=n+1}^{ny} A_{qr} B_{m-q,r-n}
%       - sum_{q=1}^{nx} sum_{r=1}^{ny-n} A_{qr} B_{m-q,n+r}
%where the second and third sub-sums have changed signs. This makes
%computation of derivatives simple, see the file header for spconv.m
%
%Input arguments:
%   T: initial guess for the solution for T (indexed as described above)
%       after the time step has been taken
%   parameters: parameter structue, must have the fields
%       G: the Green's function transform G_mn, indexed as described above
%       K_x = an nx*ny-by-nx*ny diagnoal matrix giving the value of k_xn at the
%           corresponding linear index, that is K_x = k_xm delta_ij
%           where i = n*nx+m
%       K_z = an nx*ny-by-nx*ny diagnoal matrix giving the value of k_zn at the
%           corresponding linear index that is K_z = k_zn delta_ij
%           where i = n*nx+m
%       Ra: the Rayleigh number
%       convindex: the eponymous output of convind.m for inputs nx, ny
%       dt: time step
%       T_prev: the previous solution T_mn^prev, written as a vector
%           indexed in the same way as T

%physical parameters
G = parameters.G;    %Green's function for Laplacian for P
Kx = parameters.K_x; %diagonal matrix for multiplication by x-wavenumber
Kz = parameters.K_z; %diagonal matrix for multiplication by z-wavenumber
Ra = parameters.Ra; %Rayleigh number

%convolution indexing matrices
convindex = parameters.convindex;

%time step
dt = parameters.dt;

%solution from previous time step
Tprev = parameters.Tprev;

P = -G*Kz*T;
Px = Kx*P;
Pz = Kz*P;
Tx = Kx*T;
Tz = Kz*T;

%nonlinear term convolutions
Tconv = spconv(T,convindex,-1);
Txconv =  spconv(Tx,convindex,-1);
Pzconv =  spconv(Pz,convindex,-1);
TTz = Tconv*Tz;
PxTx = Txconv*Px;
PzTz = Pzconv*Tz;

%Output
Fout = (T-Tprev)/dt  - T - Pz + TTz + PxTx + PzTz + 1/Ra*(Kx^2 + Kz^2)*T;

if nargout == 2
    DP = -G*Kz;
    DPx = Kx*DP;
    DPz = Kz*DP;
    DTx = Kx;
    DTz = Kz;
    Tzconv = spconv(Tz,convindex,+1);
    Pxconv = spconv(Px,convindex,+1);
    DTTz =  Tconv*DTz + Tzconv;
    DPxTx = Txconv*DPx + Pxconv*DTx;
    DPzTz = Pzconv*DTz + Tzconv*DPz;
    DFout = eye(length(T))/dt - eye(length(T)) - DPz + DTTz + DPxTx + DPzTz + 1/Ra*(Kx^2+Kz^2);
end

end

