function [convindex, symmindex, realindex] = convind_v2(nx,ny)
%[convindex, symmindex, realindex] = convind(nx,ny)
%Creates indexing vectors Ix, Iy, Iind, Jx, Jy, Jind, Kx, Ky, Kind such that
% SI_mn = sum_{q=1}^{nx} sum_{r=1}^{n-1} A_{qr} B_{m0+m-q,n-r}
% SJ_mn = sum_{q=1}^{nx} sum_{r=n+1}^{ny} A_{qr} B_{m0+m-q,r-n}
% SK_mn = sum_{q=1}^{nx} sum_{r=1}^{ny-n} A_{qr} B_{m0+m-q,n+r}
%can be written in the form
% Bvec =reshape(B,nx*ny,1);
% Avec = reshape(A,nx*ny,1)
% SI = reshape( sparse(Ix,Iy,Bvec(Iind),nx*ny,nx*ny)*Avec ,nx,ny);
% SJ = reshape( sparse(Jx,Jy,Bvec(Jind),nx*ny,nx*ny)*Avec ,nx,ny);
% SK = reshape( sparse(Kx,Ky,Bvec(Kind),nx*ny,nx*ny)*Avec ,nx,ny);
%for nx-by-ny matrices A and B, where m0 = ceil((nx+1)/2) is the position
%of the centre of a Fourier transform after applying fftshift
%
%Also creates an indexing vector symmindex such that
% A_{mn} = B_{m0-m,n}
%can be written in the form
% Bvec = reshape(B,nx*ny,1)
% Avec = Bvec(symmindex)
% A = reshape(Avec, nx,ny)
%
%Lastly, for matrices A_{mn} satisfying the symmetry requirement
% A_{mn} = conj(A_{m0-m,n})
%convind_v2 also computes a set of indices that maps from a
%slightly shorter vector B with real entries to A_vec:
% A_vec(realindex.realtocomplex) = realindex.realflag*B +
% 1i*(1-realindex.realflag)*B
% A_vec = 0.5*(A_vec + conj(A_vec(symmindex)));
% A = reshape(Avec,nx,ny);
%
%In detail, the mapping from B to A_vec works as follows: if A_vec is an
%nx*ny-by-1 vector then B is an (nx-1)*ny_by-1 vector. For an index ii in
%A_vec, there is a corresponding pair of indices given by
% [mm,nn] = ind2sub([nx ny],ii);
%that is mm = mod(ii-1,nx)+1; nn = floor((ii-1)/nx)+1;
%The real and imaginary parts of Avec(ii) are assigned an entry in B if and
%only if mm > m0, with realtocomplex and realflag defined through
%if mm == m0
%   realtocomplex((nn-1)*nrealtot+1) = ii;
%   realflag((nn-1)*nrealtot+1) = 1;
%elseif if mm > mm0
%   realtocomplex((nn-1)*nrealtot+mm +1-mm0) = ii;
%   realflag((nn-1)*nrealtot+mm+1-mm0) = 1;
%   realtocomplex((nn-1)*nrealtot+nreal+mm-mm0) = ii;
%   realflag((nn-1)*nrealtot+nreal+mm-mm0) = 0;
%end
%
%Input arguments
%   nx, ny: the dimensions specified above
%Output arguments:
%   convindex:  structure with fields Ix, Iy, Iind, Jx, Jy, Jind, Kx, Ky,
%               Kind as defined above, also has fields nx and ny
%   symmindex:  vector symmindex as described above
%   realindex:  structure with field realflag and realtocopmlex that allow
%               mapping from the vector B to A_vec

%initialize output
Iind = zeros(nx*ny,1);
Ix = Iind;
Iy = Iind;
Jind = Iind;
Jx = Iind;
Jy = Iind;
Kind = Iind;
Kx = Iind;
Ky = Iind;

symmindex = Iind;

realtocomplex = zeros((nx-1)*ny,1);
realflag = zeros(size(realtocomplex));

%set summation indices for rows of sparse matrices
jj=(1:(nx*ny)).';
[qq,rr] = ind2sub([nx,ny],jj);

%k_x = 0 mode
mm0 = ceil((nx+1)/2);

%number of independent real coefficients
nrealtot = 2*(nx-mm0)+1;
nreal = nx+1-mm0;

%set counters
icount = 0;
jcount = 0;
kcount = 0;

%run through linear index vector
for ii=1:(nx*ny)
    %extract Fourier indices
    [mm,nn] = ind2sub([nx ny],ii);
    %initialize index vectors
    Iloc = jj; Jloc = jj; Kloc = jj;
    
    %take each matrix in turn
    %convolution index
    Iqq = mm0+mm-qq; Irr = nn-rr;
    Jqq = mm0+mm-qq; Jrr = rr-nn;
    Kqq = mm0+mm-qq; Krr = nn+rr;
    
    %delete out-of-range indices - I
    Iloc(Iqq<1|Iqq>nx) = [];
    Irr(Iqq<1|Iqq>nx) = [];
    Iqq(Iqq<1|Iqq>nx) = [];
    Iloc(Irr<1|Irr>ny) = [];
    Iqq(Irr<1|Irr>ny) = [];
    Irr(Irr<1|Irr>ny) = [];
    %J
    Jloc(Jqq<1|Jqq>nx) = [];
    Jrr(Jqq<1|Jqq>nx) = [];
    Jqq(Jqq<1|Jqq>nx) = [];
    Jloc(Jrr<1|Jrr>ny) = [];
    Jqq(Jrr<1|Jrr>ny) = [];
    Jrr(Jrr<1|Jrr>ny) = [];
    %K
    Kloc(Kqq<1|Kqq>nx) = [];
    Krr(Kqq<1|Kqq>nx) = [];
    Kqq(Kqq<1|Kqq>nx) = [];
    Kloc(Krr<1|Krr>ny) = [];
    Kqq(Krr<1|Krr>ny) = [];
    Krr(Krr<1|Krr>ny) = [];
    
    %Convert Fourier indexes
    Iindnew = sub2ind([nx,ny],Iqq,Irr);
    Jindnew = sub2ind([nx,ny],Jqq,Jrr);
    Kindnew = sub2ind([nx,ny],Kqq,Krr);
    
    %counter increment
    inew = length(Iindnew);
    jnew = length(Jindnew);
    knew = length(Kindnew);
    
    %Populate matrix indices
    Iind(icount+1:icount+inew) = Iindnew;
    Ix(icount+1:icount+inew) = ii;
    Iy(icount+1:icount+inew) = Iloc;
    Jind(jcount+1:jcount+jnew) = Jindnew;
    Jx(jcount+1:jcount+jnew) = ii;
    Jy(jcount+1:jcount+jnew) = Jloc;
    Kind(kcount+1:kcount+knew) = Kindnew;
    Kx(kcount+1:kcount+knew) = ii;
    Ky(kcount+1:kcount+knew) = Kloc;
    
    %update counters
    icount = icount+inew;
    jcount = jcount+jnew;
    kcount = kcount+knew;
    
    %symmetry index
    symm = 2*mm0-mm;
    
    %deal with even nx non-matched component
    symm(symm>nx) = mm(symm>nx);
    symm(symm<1) = mm(symm<1);
    symmindex(ii) = sub2ind([nx,ny],symm,nn);
    
    %real and imaginery part to complex Fourier coefficient mapping
    if mm == mm0
        realtocomplex((nn-1)*nrealtot+1) = ii;
        realflag((nn-1)*nrealtot+1) = 1;
    end
    if mm > mm0
        realtocomplex((nn-1)*nrealtot+mm +1-mm0) = ii;
        realflag((nn-1)*nrealtot+mm+1-mm0) = 1;
        realtocomplex((nn-1)*nrealtot+nreal+mm-mm0) = ii;
        realflag((nn-1)*nrealtot+nreal+mm-mm0) = 0;
    end
end

convindex.nx = nx;
convindex.ny = ny;
convindex.Iind = Iind(1:icount);
convindex.Ix = Ix(1:icount);
convindex.Iy = Iy(1:icount);
convindex.Jind = Jind(1:icount);
convindex.Jx = Jx(1:icount);
convindex.Jy = Jy(1:icount);
convindex.Kind = Kind(1:icount);
convindex.Kx = Kx(1:icount);
convindex.Ky = Ky(1:icount);

realindex.realtocomplex = realtocomplex;
realindex.realflag = realflag;

end
    