function parameters = parameter_generate_v2(nx,ny,Ra,a,dt)
%parameters = parameter_generate_v2(nx,nz,Ra,a,dt)
%Generates parameter structure parameters required by
%convection_timestep_v2
%
%Input arguments (all optional)
%   nx: number of modes in the x-direction
%   nz: number of modes in the z-direction
%   Ra: Rayleigh number
%   a:  periodicity in x-direction
%   dt: size of time step
%
%Output:
%   parameters: parameter structure needed by convection_evolve,
%       convection_timestep_v2, with fields
%       nx, ny: input variables nx,nz
%       Ra:     Rayleigh number
%       a:      periodicity in x
%       dt:     time step
%       K_x, K_z: sparse differentiation matrices as defined in file header
%               for convection_timestep_v2
%       G:      sparse Green's function matrix as defined in file header fo
%               for convection_timestep_v2
%       convindex:  sparse convolution index matrix as output convind_v2
%       symmindex:  symmetry index that maps T to its complex conjugate,
%               used if 'symmetrize' option is set in convection_evolve
%       realindex:  structure of mapping arrays realflag and realtocomplex
%               used to map from a vector of real and imaginary parts of
%               Fourier components to a vector of complex-valued Fourier
%               components as used by convection_evolve,
%               convection_timestep_v2 and convetion_steady_Ra

if nargin == 0 || isempty(nx)
    nx = 45;
end
if nargin < 2 || isempty(ny)
    ny = 45;
end
if nargin < 3 || isempty(Ra)
    Ra = 8*pi^2;
end
if nargin < 4 || isempty(a)
    a = 4;
end
if nargin < 5 || isempty(dt)
    dt = 1e-2;
end

jj=(1:(nx*ny)).';
[mm,nn] = ind2sub([nx,ny],jj);

mm0 = ceil((nx+1)/2);

k_x = 2*(mm-mm0)*pi/a;
%account for asymmetry in k_x if nx is even
if floor(nx/2) == nx/2
    k_x(1) = 0;
end
k_z = nn*pi;


parameters.nx = nx;
parameters.ny = ny;
parameters.K_x = spdiags(k_x,0,nx*ny,nx*ny);
parameters.K_z = spdiags(k_z,0,nx*ny,nx*ny);
G_inv = 1./(k_x.^2+k_z.^2);
parameters.G = spdiags(G_inv,0,nx*ny,nx*ny);
parameters.Ra = Ra;
parameters.a = a;
[parameters.convindex, parameters.symmindex, parameters.realindex] = convind_v2(nx,ny);
parameters.dt = dt;

end

